MODULE DRAMTest;
(* =========================================================================  
   DRAMTest - Tests 4164 64kbit DRAM chips

   Adapted from: 
   * Simple DRAM tester
   * Chris Osborn <fozztexx@fozztexx.com>
   * http://insentricity.com/a.cl/252
   
   Target:
     STM-32F767 Cortex-M7
     
   Tested on: 
     STM32F767 Nucleo-32 Board
   
   Version 1.0 
   (c) 2017-2018 CFB Software   
   http://www.astrobe.com  

  Permission to use, copy, modify, and/or distribute this software and its
  accompanying documentation (the "Software") for any purpose with or
  without fee is hereby granted, provided that the above copyright notice
  and this permission notice appear in all copies.
  
  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHORS DISCLAIM ALL WARRANTIES
  WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
  MERCHANTABILITY, FITNESS AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
  AUTHORS BE LIABLE FOR ANY CLAIM, SPECIAL, DIRECT, INDIRECT, OR
  CONSEQUENTIAL DAMAGES OR ANY DAMAGES OR LIABILITY WHATSOEVER, WHETHER IN
  AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE DEALINGS IN OR USE OR PERFORMANCE OF THE SOFTWARE.
  ========================================================================= *)

IMPORT GPIO, Main, MCU, SYSTEM, Timers;

CONST
  PORTA = MCU.GPIOABase;
  PORTB = MCU.GPIOBBase;
  PORTC = MCU.GPIOCBase;
  PORTE = MCU.GPIOEBase;
  PORTF = MCU.GPIOFBase;
  PORTG = MCU.GPIOGBase;

  LOW = FALSE;
  HIGH = TRUE;

  INPUT = 0;
  OUTPUT = 1;
  
  AdrCount = 8;
  AdrLimit = 0FFH; 

VAR
  A: ARRAY AdrCount OF GPIO.Pin; 
  LEDGreen, LEDRed, Din, Dout, CAS, RAS, WE: GPIO.Pin;
  mSecTimer: Timers.Timer;


PROCEDURE Blink(count: INTEGER);
VAR
  i: INTEGER;
BEGIN 
  FOR i := 0 TO count - 1 DO 
    GPIO.Put(LEDGreen, HIGH);
    Timers.Delay(mSecTimer, 200);
    GPIO.Put(LEDGreen, LOW);
    Timers.Delay(mSecTimer, 200)
  END;
  GPIO.Put(LEDGreen, LOW);
  Timers.Delay(mSecTimer, 400)
END Blink;

    
PROCEDURE Error();
BEGIN
  GPIO.Put(LEDRed, HIGH);
  WHILE TRUE DO END
END Error;


PROCEDURE SetAddress(rc: INTEGER; strobe: GPIO.Pin);
VAR
  i: INTEGER;
BEGIN
  FOR i := 0 TO AdrCount - 1 DO 
    GPIO.Put(A[i], i IN BITS(rc))
  END; 
  GPIO.Put(strobe, LOW)
END SetAddress;


PROCEDURE WriteBit(row, col: INTEGER; bit: BOOLEAN);
BEGIN
  GPIO.Put(Din, bit);
  SetAddress(row, RAS);
  GPIO.Put(WE, LOW);
  SetAddress(col, CAS);
  GPIO.Put(WE, HIGH);
  GPIO.Put(CAS, HIGH);
  GPIO.Put(RAS, HIGH);
END WriteBit;


PROCEDURE ReadBit(row, col: INTEGER; VAR bit: BOOLEAN);
BEGIN 
  SetAddress(row, RAS);
  SetAddress(col, CAS);
  GPIO.Get(Dout, bit);
  GPIO.Put(CAS, HIGH);
  GPIO.Put(RAS, HIGH);
END ReadBit;


PROCEDURE FillAlternating(start: BOOLEAN);
VAR
  row, col, i: INTEGER;
  bit, rbit: BOOLEAN;
BEGIN
  bit := start;
  FOR col := 0 TO AdrLimit DO
    FOR row := 0 TO AdrLimit DO
      WriteBit(row, col, bit);
      bit := ~bit
    END
  END;

  bit := start;
  FOR col := 0 TO AdrLimit DO
    FOR row := 0 TO AdrLimit DO
      ReadBit(row, col, rbit);
      IF bit # rbit THEN 
        Error();
      END;
      bit := ~bit
    END
  END
END FillAlternating;


PROCEDURE FillSame(bit: BOOLEAN);
VAR
  row, col: INTEGER;
  rbit: BOOLEAN;
BEGIN
  
  FOR col := 0 TO AdrLimit DO
    FOR row := 0 TO AdrLimit DO
      WriteBit(row, col, bit);
    END
  END;

  FOR col := 0 TO AdrLimit DO
    FOR row := 0 TO AdrLimit DO
      ReadBit(row, col, rbit);
      IF bit # rbit THEN 
        Error();
      END;
    END
  END 
END FillSame;


PROCEDURE Run();
VAR
  i: INTEGER;
BEGIN
  Blink(1);
  FillSame(LOW);

  Blink(2);
  FillSame(HIGH);

  Blink(3);
  FillAlternating(LOW);

  Blink(4);
  FillAlternating(HIGH);
  
  GPIO.Put(LEDGreen, HIGH)

END Run;


PROCEDURE Initialise;
VAR
  i: INTEGER;
BEGIN
  Timers.Init(mSecTimer, Timers.TIM3, Timers.MSecs);
  Timers.Delay(mSecTimer, 1);
  FOR i := 1 TO 8 DO
    GPIO.Put(RAS, LOW);
    GPIO.Put(RAS, HIGH)
  END;
END Initialise;


PROCEDURE ConfigurePin(VAR pin: GPIO.Pin; direction: INTEGER);
VAR
  config: GPIO.PortConfiguration;
BEGIN
  IF direction = INPUT THEN 
    config.mode := GPIO.Mode_In;
    config.resistors := GPIO.Pull_Up
  ELSE 
    config.mode := GPIO.Mode_Out;
    config.resistors := GPIO.Floating
  END;
  config.speed := GPIO.Speed_High;
  config.outputType := GPIO.Push_Pull;
  config.alternateFunction := GPIO.AF0; 
  GPIO.Configure(pin, config)
END ConfigurePin;


PROCEDURE MapPin(VAR pin: GPIO.Pin; port, pinNo, direction: INTEGER);
BEGIN
  GPIO.Map(port, pinNo, pin);
  ConfigurePin(pin, direction)
END MapPin;
  
   
PROCEDURE ConfigurePins;
VAR
  x: SET;
BEGIN
  (* enable clocks for GPIO *)
  SYSTEM.GET(MCU.RCC_AHB1ENR, x);
  SYSTEM.PUT(MCU.RCC_AHB1ENR, x + 
    {MCU.RCC_AHB1ENR_GPIOA} +
    {MCU.RCC_AHB1ENR_GPIOB} +
    {MCU.RCC_AHB1ENR_GPIOC} +
    {MCU.RCC_AHB1ENR_GPIOE} +
    {MCU.RCC_AHB1ENR_GPIOF} +
    {MCU.RCC_AHB1ENR_GPIOG});
  
  MapPin(A[0], PORTC,  3, OUTPUT);
  MapPin(A[1], PORTF,  5, OUTPUT);
  MapPin(A[2], PORTF,  3, OUTPUT);
  MapPin(A[3], PORTE, 13, OUTPUT);
  MapPin(A[4], PORTF, 15, OUTPUT);
  MapPin(A[5], PORTG, 14, OUTPUT); 
  MapPin(A[6], PORTF, 14, OUTPUT);
  MapPin(A[7], PORTG,  9, OUTPUT); 

  MapPin(Din,  PORTF, 13, OUTPUT);
  MapPin(Dout, PORTE, 11, INPUT);
  MapPin(CAS,  PORTE,  9, OUTPUT);
  MapPin(RAS,  PORTC,  0, OUTPUT);
  MapPin(WE,   PORTA,  3, OUTPUT);

  MapPin(LEDGreen, PORTB,   0, OUTPUT);
  MapPin(LEDRed,   PORTB,  14, OUTPUT);
  
  GPIO.Put(WE, HIGH);
  GPIO.Put(RAS, HIGH);
  GPIO.Put(CAS, HIGH);

  GPIO.Put(LEDGreen, LOW);
  GPIO.Put(LEDRed, LOW)

END ConfigurePins;
  
PROCEDURE Test(pin: GPIO.Pin);
VAR
  i: INTEGER;
BEGIN
  FOR i := 0 TO 7 DO GPIO.Put(A[i], LOW) END;
  GPIO.Put(WE, LOW);
  GPIO.Put(RAS, LOW);
  GPIO.Put(CAS, LOW);
  GPIO.Put(WE, LOW);
  GPIO.Put(Din, LOW);
  GPIO.Put(Dout, LOW);
  GPIO.Put(pin, LOW);
  Blink(5);
  GPIO.Put(pin, HIGH)
END Test;
       
BEGIN
  ConfigurePins;
  Initialise();
  (* Test(WE); *)
  Run()
END DRAMTest.





